package exemplo;

import br.com.tecpoint.api.win32.W32API.LONG_PTR;
import br.com.tecpoint.api.win32.W32API.LRESULT;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;


import br.com.tecpoint.api.DeviceParameters;
import br.com.tecpoint.api.ImagesStruct;
import br.com.tecpoint.api.VAPIInterface;

import br.com.tecpoint.api.impl.Utils;
import br.com.tecpoint.api.impl.VisionAPIImpl;
import br.com.tecpoint.api.win32.User32;
import br.com.tecpoint.api.win32.W32API.HWND;
import br.com.tecpoint.api.win32.W32API.LPARAM;
import br.com.tecpoint.api.win32.W32API.WPARAM;
import com.sun.jna.ptr.IntByReference;

public class WindowsProcedure implements User32.WNDPROC {

    HWND hwnd;
    VisionAPIImpl vapi;
    String windows_procedure_debug = "Windows Procedure ->";
    DeviceParameters devp;
    ScanCheque view;

    public WindowsProcedure(HWND hwnd, VisionAPIImpl visionAPI, DeviceParameters devParam, ScanCheque scanCheque) {
        this.hwnd = hwnd;
        User32.INSTANCE.SetWindowLong(hwnd, User32.GWL_WNDPROC, this);
        vapi = visionAPI;
        devp = devParam;
        view = scanCheque;//VisionApiNativeApp.getApplication().getVisionapiNativeView();
    }

    public int callback(HWND hWnd, int uMsg, WPARAM wParam, LPARAM lParam) {

        switch (uMsg) {
            case VAPIInterface.WM_PANINI: {

                //System.out.println(windows_procedure_debug+"VisionAPI Message");
                switch (wParam.intValue()) {
                    case VAPIInterface.WMPAR_SORTER_CONNECTED: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "Device Connected");}
                        if(view != null){
                            view.eventSorterConnected();
                        }
                        break;
                    }

                    case VAPIInterface.WMPAR_SORTER_DISCONNECTED: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "Device Disconnected");}
                        if(view != null){
                            view.eventSorterDisconnected();
                        }
                        break;
                    }

                    case VAPIInterface.WMPAR_SORTER_NEW_DOCUMENT: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "NewDocument");}
                        if(view != null){
                            view.eventNewDocument(lParam.intValue());
                        }
                        break;
                    }

                    case VAPIInterface.WMPAR_SORTER_MICR_AVAILABLE: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "MICR Available");}
                        //System.out.println(vapi.getMicrCodeline());
                        if(view != null){
                            view.eventMicroAvailable(lParam.intValue());
                        }
                        break;
                    }
                    case VAPIInterface.WMPAR_SORTER_SET_ITEM_OUTPUT: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "Set Item Output");}
                        //vapi.setPocket(lParam.intValue(), 1);
                        if(view != null){
                            view.eventSetItemOutPut(lParam.intValue());
                        }
                        break;
                    }
                    case VAPIInterface.WMPAR_SORTER_DOCUMENT_INSIDE_POCKET: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "Inside Pocket");}
                        if(view != null){
                            view.eventInsidePocket(lParam.intValue());
                        }
                        break;
                    }

                    case VAPIInterface.WMPAR_SORTER_EXCEPTION: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "Sorter Excepion");}
                        if(view != null){
                            view.eventSorterException(hWnd, uMsg, wParam, lParam);
                        }
                        break;
                    }

                    case VAPIInterface.WMPAR_IMAGES_READY: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "Images Ready");}
                        ImagesStruct.ByRef is = new ImagesStruct.ByRef();
                        //create image directory (if doesn't exist)
                        boolean bHasImage = true;
                        File snippetDir;
                        snippetDir = new File(".\\images");
                        snippetDir.mkdir();
                        byte[][] imageBytes = new byte[4][];
                        for (int i = 1; i <= 4; i++) {
                            bHasImage = true;
                            switch (i - 1) {
                                case 0: {
                                    if (devp.ImagePropertiesFront1.Format == VAPIInterface.FORMAT_NONE) {
                                        bHasImage = false;
                                    }
                                }
                                break;
                                case 1: {
                                    if (devp.ImagePropertiesFront2.Format == VAPIInterface.FORMAT_NONE) {
                                        bHasImage = false;
                                    }
                                }
                                break;
                                case 2: {
                                    if (devp.ImagePropertiesRear1.Format == VAPIInterface.FORMAT_NONE) {
                                        bHasImage = false;
                                    }
                                }
                                break;
                                case 3: {
                                    if (devp.ImagePropertiesRear2.Format == VAPIInterface.FORMAT_NONE) {
                                        bHasImage = false;
                                    }
                                }
                                break;
                            }
                            //has Image?
                            if (bHasImage) {
                                //save ?
                                imageBytes[i - 1] = Utils.getImageByteFormLparamIntValue(lParam.intValue(), i, is);
                                if (view.isSaveImages()) {
                                    FileOutputStream fos;
                                    try {
                                        //fos = new FileOutputStream(".\\images\\Doc" + is.DocNumber + "Image" + i +".bmp");
                                        fos = new FileOutputStream(".\\images\\Doc" + is.DocNumber + "Image" + i + Utils.getExtension(devp.ImagePropertiesFront1.Format));
                                        fos.write(imageBytes[i - 1]);
                                        fos.close();
                                    } catch (IOException e) {
                                        // TODO Auto-generated catch block
                                        e.printStackTrace();
                                    }
                                }
                                //Show
                                if (view != null) {
                                    view.eventImageReady(i - 1, imageBytes[i - 1], Utils.getJaiFileFormatName(devp.ImagePropertiesFront1.Format));
                                }
                            }
                        }
                        vapi.freeImagesBuffer(new IntByReference(lParam.intValue()).getPointer().getPointer(0));
                        break;
                    }
                    case VAPIInterface.WMPAR_SNIPPETS_READY: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "Snippets Ready");}
                        ImagesStruct.ByRef is = new ImagesStruct.ByRef();
                        //create snippet directory (if doesn't exist)
                        File snippetDir;
                        boolean bSave = true;
                        snippetDir = new File(".\\snippets");
                        snippetDir.mkdir();
                        byte[][] imageBytes = new byte[10][];
                        for (int i = 1; i <= 10; i++) {

                            switch (i - 1) {
                                case 0: {
                                    if (devp.SnippetProperties0.Enable == VAPIInterface.TRUE) {
                                        bSave = false;
                                    }
                                }

                                break;
                                case 1: {
                                    if (devp.SnippetProperties1.Enable == VAPIInterface.TRUE) {
                                        bSave = false;
                                    }
                                }
                                break;
                                case 2: {
                                    if (devp.SnippetProperties2.Enable == VAPIInterface.TRUE) {
                                        bSave = false;
                                    }
                                }
                                break;
                                case 3: {
                                    if (devp.SnippetProperties3.Enable == VAPIInterface.TRUE) {
                                        bSave = false;
                                    }
                                }
                                break;
                                case 4: {
                                    if (devp.SnippetProperties4.Enable == VAPIInterface.TRUE) {
                                        bSave = false;
                                    }
                                }
                                break;
                                case 5: {
                                    if (devp.SnippetProperties5.Enable == VAPIInterface.TRUE) {
                                        bSave = false;
                                    }
                                }
                                break;
                                case 6: {
                                    if (devp.SnippetProperties6.Enable == VAPIInterface.TRUE) {
                                        bSave = false;
                                    }
                                }
                                break;
                                case 7: {
                                    if (devp.SnippetProperties7.Enable == VAPIInterface.TRUE) {
                                        bSave = false;
                                    }
                                }
                                break;

                                case 8: {
                                    if (devp.SnippetProperties8.Enable == VAPIInterface.TRUE) {
                                        bSave = false;
                                    }
                                }
                                break;
                                case 9: {
                                    if (devp.SnippetProperties9.Enable == VAPIInterface.TRUE) {
                                        bSave = false;
                                    }
                                }
                                break;


                            }
                            imageBytes[i - 1] = Utils.getImageByteFormLparamIntValue(lParam.intValue(), i, is);
                            if (bSave) {
                                FileOutputStream fos;
                                try {
                                    if (is.SnippetFront) {
                                        fos = new FileOutputStream(".\\snippets\\Doc" + is.DocNumber + "SnippetsFront" + i + ".bmp");
                                    } else {
                                        fos = new FileOutputStream(".\\snippets\\" + is.DocNumber + "SnippetsRear" + i + ".bmp");
                                    }
                                    fos.write(imageBytes[i - 1]);
                                    fos.close();
                                } catch (IOException e) {
                                    // TODO Auto-generated catch block
                                    e.printStackTrace();
                                }
                            }

                        }

                        vapi.freeSnippetBuffer(new IntByReference(lParam.intValue()).getPointer().getPointer(0));
                        break;
                    }
                    case VAPIInterface.WMPAR_DOC_COMPLETED: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "Doc Completed");}
                        view.eventDocCompleted(lParam.intValue());
                        break;
                    }
                    case VAPIInterface.WMPAR_MAGCARD: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "MagCard");}
                        view.eventMagCard();
                        break;
                    }
                    case VAPIInterface.WMPAR_SET_DOWNSTREAM_OPTIONS: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "Set DownStream Options");}
                        view.eventSetDownStreamOptions();
                    }
                    case VAPIInterface.WMPAR_COMPRESSED_IMG_FOR_DECISION: {
                        if(vapi.isDebugMode()){System.out.println(windows_procedure_debug + "Compressed Img for Decision");}
                        view.eventCompressedImgForDecision();
                        break;
                    }
                    default: {
                        /*System.out.println("Windows Message = " + wParam.intValue());
                        System.out.println("====================================");
                        System.out.println(hWnd);
                        System.out.println(uMsg);
                        System.out.println(wParam);
                        System.out.println(lParam);
                        System.out.println("====================================");*/
                        break;
                    }
                }
                break;
            }
            case User32.WM_DESTROY: {
                vapi.shutDown();
                break;
            }

            default: {
                /*System.out.println("Windows Message = " + wParam.intValue());
                System.out.println("====================================");
                System.out.println(hWnd);
                System.out.println(uMsg);
                System.out.println(wParam);
                System.out.println(lParam);
                System.out.println("====================================");*/
                break;

            }
        }
        return User32.INSTANCE.DefWindowProc(hWnd, uMsg, wParam, lParam);
    }

    public LRESULT callWindowProc(LONG_PTR arg0, HWND hWnd, int uMsg, WPARAM wParam, LPARAM lParam) {
        //return User32.INSTANCE.CallWindowProc(arg0, hWnd, uMsg, wParam, lParam);
        throw new UnsupportedOperationException("Not supported yet.");
    }


}